#!/bin/bash
export LANG=en_US.UTF-8
_stage=$1
_stage_lock="/root/initial/tasks/${_stage}.done"
_prov_type=$(dmidecode -s system-product-name)
_centos_release=$(/bin/rpm --eval='%centos_ver')
if [ "${_prov_type}" == 'OpenStack Nova' ]; then
  _prov_type=openstack
else
  _prov_type=kickstart
fi

stage1() {
    check_stage
    if [ "${_prov_type}" == 'openstack' ]; then
      sed -i '1 i\MAILTO=""' /var/spool/cron/root
      touch /var/cpanel/activate/features/disable_feature_showcase
      touch /etc/.whostmgrft
      sed -i '\#/dev/shm#d' /etc/fstab
      cat >> /etc/fstab <<-FSTAB_EOF
	tmpfs   /dev/shm        tmpfs   rw,noexec,nosuid,noatime,nodev,mode=711,size=500M       0       0
	tmpfs   /ramdisk        tmpfs   defaults,noatime,nosuid,nodev,size=4096m                0       0
	none    /var/spool/exim tmpfs   rw,noexec,nosuid,nodev,uid=47,gid=12,mode=750           0       0
	FSTAB_EOF
  
      ## Creates directories and starts needed services for multihome
      mkdir -vp /{admin,ramdisk}
      curl -s http://kickstart.hostgator.com/hgstartprovo > /etc/init.d/hgstartprovo
      mount -a
      chmod +x /etc/init.d/hgstartprovo
      chkconfig hgstartprovo on
      service hgstartprovo start
      yum install epel-release -y
      yum install https://yum.puppetlabs.com/puppetlabs-release-el-6.noarch.rpm -y
      yum install puppet -y
    fi

    /usr/bin/curl -s "http://nas.hostgator.com/assign.php?server=$(uname -n)"
    if [ "${_centos_release}" -eq 7 ]; then
      timedatectl set-timezone "${TIME_ZONE}"
      timedatectl set-local-rtc 0
      timedatectl set-ntp true
      _mysql_service=mysqld
    else
      rm -vf /etc/localtime
      ln -vs "/usr/share/zoneinfo/${TIME_ZONE}" /etc/localtime
      _mysql_service=mysql
    fi
    sed -i '/mysql/d' /etc/chkserv.d/chkservd.conf
    service cpanel stop
    service ${_mysql_service} stop
    mkdir -vp /opt/eig_linux/etc/puppetfeatures
    touch /opt/eig_linux/etc/puppetfeatures/no_custom_jaillshell
    rm -vf {/var/cpanel/globalcache/cpanel.cache,/root/.cpanel/datastore/MOUNTS_CACHE__proc_mounts}
    FACTER_kernelrelease=lve FACTER_foreman_organization=${BRAND} FACTER_foreman_location=${LOCATION} puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags foreman_registration
    while [ $? -eq 1 ]; do
        sleep 5
        FACTER_kernelrelease=lve FACTER_foreman_organization=${BRAND} FACTER_foreman_location=${LOCATION} puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags foreman_registration
    done
    touch "${_stage_lock}"
}

stage2() {
    check_stage
    puppet agent --enable
    FACTER_kernelrelease=lve puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags=yum_repos,puppet_agent,resolvers
    while [ $? -eq 1 ]; do
        sleep 5
        FACTER_kernelrelease=lve puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags=yum_repos,puppet_agent,resolvers
    done
    puppet agent --disable 'provisioning in progress'
    yum clean all -y --enablerepo=*
    yum makecache
    if [ -f /etc/cpanel/ea4/is_ea4 ]; then
        if [ "${_centos_release}" -eq 7 ]; then
          sed -i '/JoinControllers/c\JoinControllers=blkio,cpu,cpuacct,cpuset,memory' /etc/systemd/system.conf
          source /root/initial/ea4_c7_rpms
	  _perc_repo=eig-percona57
	  rm -vfr /var/lib/mysql
          mkdir -vp /etc/systemd/system/{httpd.service.d,cpanel.service.d,dovecot.service.d,mysqld.service.d}
	  touch /opt/eig_linux/etc/puppetfeatures/esop17141
        else
          source /root/initial/ea4_c6_rpms
          _perc_repo=hgpercona56
        fi
        mkdir -vp /etc/cpanel/ea4/profiles/custom
    else
        source /root/initial/rpms
        _perc_repo=hgpercona56
    fi
    brand_rpms=${BRAND}_rpms
    prov_rpms=${_prov_type}_rpms
    prod_rpms=${PRODUCT}_rpms
    yum -y remove ${rm_rpms}
    yum -y --enablerepo=${_perc_repo} --enablerepo=hgrepo-testing --enablerepo=epel --disableexcludes=main install ${common_rpms} ${!prov_rpms} ${!prod_rpms} ${!brand_rpms} || exit 1

    if [ ! -f /etc/cpanel/ea4/is_ea4 ]; then
        ## Fix PHP symlinks
	if [ "${PRODUCT}" == 'owp' ]; then
          _phpver=71
	else
          _phpver=56
        fi	  
        for bin in php pear peardev pecl php-cgi php-config phpize; do
            for bin_path in /usr/local/bin /usr/bin; do
                if [[ ! -L ${bin_path}/${bin} ]]; then
                    if [[ -e ${bin_path}/${bin} ]]; then
                        mv -v ${bin_path}/${bin} ${bin_path}/${bin}-$(date +%s)
                    fi
                    if [[ -e /opt/php${_phpver}/bin/${bin} ]]; then
                        ln -fvs /opt/php${_phpver}/bin/${bin} ${bin_path}/${bin}
                    fi
                fi
            done
        done

        #### Fix symlink. Original /usr/bin/php is the CGI version not CLI
        rm -vf /usr/bin/php
        ln -vs /opt/php${_phpver}/bin/php-cgi /usr/bin/php
      
        if [ -x /usr/bin/php-cli ]; then
            mv -v /usr/bin/php-cli /usr/bin/php-cli-$(date +%s)
            ln -vs /opt/php${_phpver}/bin/php /usr/bin/php-cli
        fi
    fi
    touch "${_stage_lock}"
}

stage3() {
    check_stage
    chkconfig datacycle on
    chkconfig eig-firewall on
    service ipaliases start
    touch /etc/mysqlupdisable
    touch /etc/ftpupdisable
    if [ ! -f /etc/cpanel/ea4/is_ea4 ]; then
        touch /var/cpanel/httpd_was_built_by_ea3
    fi
    service mysql status &> /dev/null || service mysql start
    /scripts/upcp --sync
    service mysql status &> /dev/null || service mysql start
    sed -i '/\(upcp --cron\|optimizefs\|update_db_cache\)/d' /var/spool/cron/root
    if [ "${PRODUCT}" != 'owp' ]; then
      if [[ "${BRAND}" == 'HOSTGATOR' ]]; then
          yum install -y quickinstall-hg --enablerepo=hgrepo-testing
      elif [[ ! "${BRAND}" =~ ^(BLUEHOST|LATAM)$ ]]; then
          yum install -y quickinstall --enablerepo=hgrepo-testing
      fi
      ### Add postgres for BH ESOP-2655
      if [ "${BRAND}" == "BLUEHOST" ]; then
          echo 'y' | /scripts/installpostgres
          /opt/eig_linux/bin-legacy/fixpostgres
      fi
    fi
    puppet agent --enable
    FACTER_kernelrelease=lve puppet agent -t
    while [ $? -eq 1 ]; do
        sleep 5
        FACTER_kernelrelease=lve puppet agent -t 
    done
    service mysql status &> /dev/null || service mysql start
    FACTER_kernelrelease=lve puppet agent -t
    service mysql status &> /dev/null || service mysql start
    FACTER_kernelrelease=lve puppet agent -t
    if [ -f /etc/cpanel/ea4/is_ea4 ]; then
        _phpver=ea-php72
        if [ "${PRODUCT}" == 'owp' ]; then
	  _php_fpm=1
        fi
        /usr/local/bin/ea_install_profile --install /etc/cpanel/ea4/profiles/custom/ea4_eig_profile.json || exit 1
	_php_vers=( $(/bin/awk -F: '/suphp/ {print $1}' <(/usr/local/cpanel/bin/rebuild_phpconf --available)) )
        /usr/local/cpanel/bin/rebuild_phpconf --default=${_phpver}$(printf -- ' --%s=suphp' "${_php_vers[@]}") || exit 1
	if [ "${_php_fpm}" == 1 ]; then
          whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm=1
	  /scripts/php_fpm_config --convert_all --noprompt
	fi
    fi
    rm -vf /var/cpanel/ssl/*-{CRTINFO,NOT_AFTER,CN,SIGNATURE_CHAIN_VERIFIED,DOMAINS}
    /usr/local/cpanel/bin/checkallsslcerts
    /scripts/install_lets_encrypt_autossl_provider
    whmapi1 set_autossl_provider provider=LetsEncrypt x_terms_of_service_accepted=https://acme-v01.api.letsencrypt.org/terms
    whmapi1 set_autossl_metadata metadata_json='{"clobber_externally_signed":1, "notify_autossl_expiry":0, "notify_autossl_expiry_coverage":0, "notify_autossl_renewal":0, "notify_autossl_renewal_coverage":0, "notify_autossl_renewal_coverage_reduced":0, "notify_autossl_renewal_uncovered_domains":0}'
    /opt/hgmods/multi_shared.pl
    touch /var/cpanel/diya_allow
    rm -vf /admin/MIRRORING.FROM
    yum -y --enablerepo=endurance-414-kernel-testing update
    chkconfig fastmail off
    /scripts/fixquotas
    rm -vf /var/cpanel/conf/apache/local
    /scripts/rebuildhttpdconf
    mysql_upgrade

    ## Add Backups
    _min=$(shuf -i 0-59 -n 1)
    _hour=$(shuf -i 0-23 -n 1)
    _day=$( head -1 <(grep -oa '[0-1]\|[5-6]' /dev/urandom) )

    echo "${_min} ${_hour} * * ${_day} cd /admin/hgbackupdir/; ./backup.pl --full" >> /var/spool/cron/root
    echo "${_min} ${_hour} * * 3 cd /admin/hgbackupdir/; ./backup.pl --innodb" >> /var/spool/cron/root

    ### custapi randomly drops requests causing it to fail. Adding this as a work around
    /opt/hgmods/hal_accesshash.pl
    while [ $? -eq 1 ]; do
        sleep 5
        /opt/hgmods/hal_accesshash.pl
    done
    sed -i -r '/\/(home[1-9]?|mysql)/s/[0-9]$/0/' /etc/fstab
    touch "${_stage_lock}"
}

stage4() {
    check_stage
    #cp_update
    yum -y remove initial-scripts-${BRAND}
    echo "Rebooting in 2 Minutes"
    echo "shutdown -rf now" | at now + 2 min
    touch "${_stage_lock}"
}

cp_update() {
    if ! grep -q '11.58' /usr/local/cpanel/version; then
        touch /opt/eig_linux/etc/puppetfeatures/cpupdate1158
        FACTER_kernelrelease=lve puppet agent -t --tags=cpanel
        FACTER_kernelrelease=lve puppet agent -t --tags=cpanel
        /scripts/upcp
    fi
}

check_stage() {
    if [ -f "${_stage_lock}" ]; then
        exit 0
    fi
    echo "Running ${_stage}"
}

${_stage}
